/*
Copyright © 2023 Philipp Wolfer <phw@uploadedlobster.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package models

import (
	"time"

	"github.com/spf13/viper"
)

// A listen service backend.
// All listen services must implement this interface.
type Backend interface {
	FromConfig(config *viper.Viper) Backend
}

// Must be implemented by services supporting the export of listens.
type ListensExport interface {
	// Returns a list of all listens newer then oldestTimestamp.
	// The returned list of listens is supposed to be ordered by the
	// Listen.ListenedAt timestamp, with the oldest entry first.
	ExportListens(oldestTimestamp time.Time) ([]Listen, error)
}

// Must be implemented by services supporting the import of listens.
type ListensImport interface {
	// Imports the given list of listens.
	ImportListens(listens []Listen, oldestTimestamp time.Time) (ImportResult, error)
}

// Must be implemented by services supporting the export of loves.
type LovesExport interface {
	// Returns a list of all loves newer then oldestTimestamp.
	// The returned list of listens is supposed to be ordered by the
	// Love.Created timestamp, with the oldest entry first.
	ExportLoves(oldestTimestamp time.Time) ([]Love, error)
}

// Must be implemented by services supporting the import of loves.
type LovesImport interface {
	// Imports the given list of loves.
	ImportLoves(loves []Love, oldestTimestamp time.Time) (ImportResult, error)
}

type ImportResult struct {
	TotalCount    int
	ImportCount   int
	LastTimestamp time.Time
	ImportErrors  []string
}
